@@ -11,29 +11,6 @@ use Cache::Simple::TimedExpiry;
 
 use strict;
 
-# {{{ Doc
-
-=head1 NAME
-
-DBIx::SearchBuilder::Record::Cachable - Records with caching behavior
-
-=head1 SYNOPSIS
-
-  package MyRecord;
-  use base qw/DBIx::SearchBuilder::Record::Cachable/;
-
-=head1 DESCRIPTION
-
-This module subclasses the main DBIx::SearchBuilder::Record package to add a caching layer. 
-
-The public interface remains the same, except that records which have been loaded in the last few seconds may be reused by subsequent fetch or load methods without retrieving them from the database.
-
-=head1 METHODS
-
-=cut
-
-# }}}
-
 my %_CACHES = ();
 
 # Function: new
@@ -45,6 +22,17 @@ sub new () {
     my ( $class, @args ) = @_;
     my $self = $class->SUPER::new(@args);
 
+    $self->{'_Class'} =
+      ref($self);    # Cache it since we're gonna look at it a _lot_
+
+    if ( $self->can('_CacheConfig') ) {
+        $self->{'_CacheConfig'} = $self->_CacheConfig();
+    }
+    else {
+        $self->{'_CacheConfig'} = __CachableDefaults::_CacheConfig();
+    }
+
+
     return ($self);
 }
 
@@ -52,47 +40,22 @@ sub _SetupCache {
     my $self  = shift;
     my $cache = shift;
     $_CACHES{$cache} = Cache::Simple::TimedExpiry->new();
-    $_CACHES{$cache}->expire_after( $self->_CacheConfig->{'cache_for_sec'} );
+    $_CACHES{$cache}->expire_after( $self->{'_CacheConfig'}{'cache_for_sec'} );
 }
 
-=head2 FlushCache 
-
-This class method flushes the _global_ DBIx::SearchBuilder::Record::Cachable 
-cache.  All caches are immediately expired.
-
-=cut
-
-sub FlushCache {
-    %_CACHES = ();
-}
-
-
 sub _KeyCache {
     my $self = shift;
-    my $cache = $self->_Handle->DSN . "-KEYS--" . ($self->{'_Class'} ||= ref($self));
-    $self->_SetupCache($cache) unless exists ($_CACHES{$cache});
-    return ($_CACHES{$cache});
+    my $cache =     $self->_Handle->DSN . "-KEYS--" . $self->{'_Class'};
+    $self->_SetupCache($cache)  unless exists ($_CACHES{$cache});
+    return ( $_CACHES{ $cache});
 
 }
 
-=head2 _FlushKeyCache
-
-Blow away this record type's key cache
-
-=cut
-
-
-sub _FlushKeyCache {
-    my $self = shift;
-    my $cache = $self->_Handle->DSN . "-KEYS--" . ($self->{'_Class'} ||= ref($self));
-    $self->_SetupCache($cache);
-}
-
 sub _RecordCache {
     my $self = shift;
-    my $cache = $self->_Handle->DSN . "--" . ($self->{'_Class'} ||= ref($self));
-    $self->_SetupCache($cache) unless exists ($_CACHES{$cache});
-    return ($_CACHES{$cache});
+    my $cache =     $self->_Handle->DSN . "--" . $self->{'_Class'};
+    $self->_SetupCache($cache)  unless exists ($_CACHES{$cache});
+    return ( $_CACHES{ $cache});
 
 }
 
@@ -166,12 +129,12 @@ sub __Set () {
 # Args    : nil
 # Lvalue  : ?
 
-sub __Delete () {
+sub Delete () {
     my ($self) = @_;
 
     $self->_expire();
 
-    return $self->SUPER::__Delete();
+    return $self->SUPER::Delete();
 
 }
 
@@ -184,9 +147,6 @@ sub __Delete () {
 sub _expire (\$) {
     my $self = shift;
     $self->_RecordCache->set( $self->_primary_RecordCache_key , undef, time-1);
-    # We should be doing something more surgical to clean out the key cache. but we do need to expire it
-    $self->_FlushKeyCache;
-   
 }
 
 # Function: _fetch
@@ -198,7 +158,6 @@ sub _expire (\$) {
 sub _fetch () {
     my ( $self, $cache_key ) = @_;
     my $data = $self->_RecordCache->fetch($cache_key) or return;
-
     @{$self}{keys %$data} = values %$data; # deserialize
     return 1;
 
@@ -228,7 +187,7 @@ sub _serialize {
     return (
         {
             values  => $self->{'values'},
-            table   => $self->Table,
+            table   => $self->{'table'},
             fetched => $self->{'fetched'}
         }
     );
@@ -242,8 +201,7 @@ sub _serialize {
 
 sub _gen_alternate_RecordCache_key {
     my ( $self, %attr ) = @_;
-    #return( Storable::nfreeze( %attr));
-   my $cache_key;
+    my $cache_key = $self->Table() . ':';
     while ( my ( $key, $value ) = each %attr ) {
         $key   ||= '__undef';
         $value ||= '__undef';
@@ -267,7 +225,7 @@ sub _gen_alternate_RecordCache_key {
 
 sub _fetch_RecordCache_key {
     my ($self) = @_;
-    my $cache_key = $self->_CacheConfig->{'cache_key'};
+    my $cache_key = $self->{'_CacheConfig'}{'cache_key'};
     return ($cache_key);
 }
 
@@ -323,17 +281,7 @@ sub _lookup_primary_RecordCache_key {
 
 }
 
-=head2 _CacheConfig 
-
-You can override this method to change the duration of the caching from the default of 5 seconds. 
-
-For example, to cache records for up to 30 seconds, add the following method to your class:
-
-  sub _CacheConfig {
-      { 'cache_for_sec' => 30 }
-  }
-
-=cut
+package __CachableDefaults;
 
 sub _CacheConfig {
     {
@@ -341,22 +289,4 @@ sub _CacheConfig {
         'cache_for_sec' => 5,
     };
 }
-
 1;
-
-__END__
-
-# {{{ POD
-
-=head1 AUTHOR
-
-Matt Knopp <mhat@netlag.com>
-
-=head1 SEE ALSO
-
-L<DBIx::SearchBuilder>, L<DBIx::SearchBuilder::Record>
-
-=cut
-
-# }}}
-
